/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * DlgArrow.cpp
 *  Created on: 17.10.2009
 */

#include "DlgArrow.h"
#include <QtGui>
#include <QVBoxLayout>
#include <QSpacerItem>
#include "DlgArrowModelSimplified.h"
#include "DlgArrowModelFull.h"


DlgArrow::DlgArrow(QWidget * parent,  Qt::WFlags f )
: QDialog(parent, f)
{

	mModelDlg = NULL;
	mCancel = NULL;
	mAccept = NULL;

}


void DlgArrow::init(CPfeil &p, UnitConverter& unitconv)
{
	m_arrow = p;
	m_unitconverter = unitconv;


	mModelDlg = new QStackedWidget;

	if (!m_arrow.m_data.contains("txtModel")) {
		QMap<QString, QVariant> map;
		// Default for new Arrows are "simplified"
		// A new Arrow without data:
		m_arrow.setValue("txtModel/status", Datastore::user);
		if(m_arrow.m_data.size() < 3 ) {
			m_arrow.setValue("txtModel/value", QString("simplified"));
		}else{
			m_arrow.setValue("txtModel/value", QString("full")); // if the map contains data, the Default Model is full, for backward compatibility:
		}
	}

	// Create the subwidgets for the StackedWidget
	DlgArrowModelFull *full = new DlgArrowModelFull;
	full->init(&m_arrow, unitconv);
	mModelDlg->addWidget(full);

	DlgArrowModelSimplified *simplified = new DlgArrowModelSimplified;
	simplified->init(&m_arrow, unitconv);
	mModelDlg->addWidget(simplified);

	mCancel = new QPushButton(tr("cancel"));
	mAccept = new QPushButton(tr("ok"));

	mName = new QLineEdit(p.m_data["title"].toString());

	mModel = new QButtonGroup();
	mModelFullArrow = new QRadioButton(tr("Full Arrow"));
	mModelFullArrow->setToolTip(tr("This model is mainly for arrows. Use this model for side wind"));
	mModelSimplifiedArrow = new QRadioButton(tr("Simplified Arrow"));
	mModelSimplifiedArrow->setToolTip(tr("This model need only 2 values. It can also used for bullets."));
	mModel->addButton(mModelFullArrow);
	mModel->addButton(mModelSimplifiedArrow);


	QMap<QString, QVariant> map = m_arrow.m_data["txtModel"].toMap();
	if(map["value"] == QString("full")) {
		mModelFullArrow->setChecked(true);
		mModelDlg->setCurrentIndex(0);
	}
	if(map["value"] == QString("simplified")) {
		mModelSimplifiedArrow->setChecked(true);
		mModelDlg->setCurrentIndex(1);
	}

	QLabel *lblName = new QLabel(tr("Name"));
	//QSpacerItem *horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
	QSpacerItem *horizontalSpacer_1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

	QFormLayout *formlayout = new QFormLayout;
	QVBoxLayout *mainlayout = new QVBoxLayout;
	QHBoxLayout *footerlayout = new QHBoxLayout;

	formlayout->addRow(tr("Name"), mName);
	formlayout->addRow(tr("Model"), mModelSimplifiedArrow);
	formlayout->addRow("", mModelFullArrow);

	footerlayout->addWidget(mCancel);
	footerlayout->addWidget(mAccept);
	footerlayout->addSpacerItem(horizontalSpacer_1);

	mainlayout->addLayout(formlayout);
	mainlayout->addWidget(mModelDlg);
	mainlayout->addLayout(footerlayout);

	this->setLayout(mainlayout);

	connect(mAccept, SIGNAL(clicked()), this, SLOT(accept()));
	connect(mCancel, SIGNAL(clicked()), this, SLOT(reject()));

	connect(mModelFullArrow, SIGNAL(toggled(bool)), this, SLOT(on_mModelFullArrow_toggled(bool)));
	connect(mModelSimplifiedArrow, SIGNAL(toggled(bool)), this, SLOT(on_mModelSimplifiedArrow_toggled(bool)));

}


void DlgArrow::accept()
{
	m_arrow.m_data["title"] = mName->text();
	QDialog::accept();
}

void DlgArrow::on_mModelFullArrow_toggled ( bool checked )
{
	m_arrow.setValue("txtModel/status", Datastore::user);

	if(checked) {
			m_arrow.setValue("txtModel/value", QString("full")); // if the map contains data, the Default Model is full, for backward compatibility:
			mModelDlg->setCurrentIndex(0);

	}else{
			m_arrow.setValue("txtModel/value", QString("simplified"));
	}
	//m_arrow.m_data["txtModel"] = map;
	if(mModelDlg != NULL) {
		DlgArrowModel *w = dynamic_cast<DlgArrowModel*>(mModelDlg->currentWidget());
		w->slotUpdateAllViews(NULL);
	}
}
void DlgArrow::on_mModelSimplifiedArrow_toggled ( bool checked )
{
	QMap<QString, QVariant> map = m_arrow.m_data["txtModel"].toMap();
	//m_arrow.m_data = QMap<QString, QVariant>(); // delete the map
	map["status"] = DlgArrowModel::modified_by_user;

	if(checked) {
		m_arrow.setValue("txtModel/value", QString("simplified"));
		mModelDlg->setCurrentIndex(1);


	}else{
		m_arrow.setValue("txtModel/value", QString("full")); // if the map contains data, the Default Model is full, for backward compatibility:
	}
	if(mModelDlg != NULL) {
		DlgArrowModel *w = dynamic_cast<DlgArrowModel*>(mModelDlg->currentWidget());
		w->slotUpdateAllViews(NULL);
	}

	//m_arrow.m_data["txtModel"] = map;

}
