/***************************************************************************
 *   Copyright (C) 2011 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Pfeil.h"
#include <ballistic/ballistic.h>
#include <ballistic/utility.h>
#include "arrowmatcher_file.h"
#include <QDebug>
#include <QApplication>
using namespace arw;

CPfeil::CPfeil()  {
	this->init();

}
/** construct a arrow with the same values as a, also the guid
 * @param CPfeil a   the arrow to copy
 *
 */
CPfeil::CPfeil(const CPfeil &a) : Datastore(a)
{
	//qDebug() << __LINE__ << " " << __FILE__ << this->toString();
    //setValue("", a.m_data);
	this->take_data(a);
	//qDebug() << __LINE__ << " " << __FILE__ << this->toString();

}
/**
* \brief create a new CPfeil with the guid
*
* \param QString  guid
*/
CPfeil::CPfeil(const QString guid) : Datastore(guid) {
	this->init();

}


CPfeil & CPfeil::take_data(const CPfeil & p){
	 if(this == &p)
	        return *this;

	cp = p.cp;
	J = p.J;
	m = p.m;
	A_frontal = p.A_frontal;
	A_cross = p.A_cross;
	ff_frontal = p.ff_frontal;
	ff_cross = p.ff_cross;
	Datastore::take_data(p);
	//qDebug() << __LINE__ << " " << __FILE__ << this->toString();
	return *this;

}
bool CPfeil::setValue(QString path, QVariant value) {
	bool ok = Datastore::setValue(path, value);
	if(!ok)
		return false;

	UnitConverter uv;
	double v;
	QString unitstr;

	QString model = this->getValue("txtModel/value").toString();
	if(QString("simplified") == model) {
		Datastore::setValue("txtFF_Cross/hidden", 1);
		Datastore::setValue("txtAreaCrossSection/hidden", 1);
		Datastore::setValue("txtCenterOfPresure/hidden", 1);
		Datastore::setValue("txtMomentOfInertia/hidden", 1);

		Datastore::setValue("txtShaftweight/hidden", 1);
		Datastore::setValue("txtShaftlength/hidden", 1);
		Datastore::setValue("txtShaftcode/hidden", 1);
		Datastore::setValue("txtShaftthickness/hidden", 1);
		Datastore::setValue("txtShaftdisplacement/hidden", 1);
		Datastore::setValue("txtFletchesarea/hidden", 1);
		Datastore::setValue("txtFletcheslength/hidden", 1);
		Datastore::setValue("txtFletchesweight/hidden", 1);
		Datastore::setValue("txtFletchesdisplacement/hidden", 1);
		Datastore::setValue("txtPointweight/hidden", 1);
		Datastore::setValue("txtPointdisplacement/hidden", 1);
		Datastore::setValue("txtNockweight/hidden", 1);
		Datastore::setValue("txtNockdisplacement/hidden", 1);




		Datastore::setValue("txtShaftdia/hidden", 0);


	}
	if(QString("full") == model) {
		Datastore::setValue("txtFF_Cross/hidden", 0);
		Datastore::setValue("txtAreaCrossSection/hidden", 0);
		Datastore::setValue("txtCenterOfPresure/hidden", 0);
		Datastore::setValue("txtMomentOfInertia/hidden", 0);

		Datastore::setValue("txtShaftdia/hidden", 0);


		Datastore::setValue("txtShaftweight/hidden", 0);
		Datastore::setValue("txtShaftlength/hidden", 0);
		Datastore::setValue("txtShaftcode/hidden", 0);
		Datastore::setValue("txtShaftthickness/hidden", 0);
		Datastore::setValue("txtShaftdisplacement/hidden", 0);
		Datastore::setValue("txtFletchesarea/hidden", 0);
		Datastore::setValue("txtFletcheslength/hidden", 0);
		Datastore::setValue("txtFletchesweight/hidden", 0);
		Datastore::setValue("txtFletchesdisplacement/hidden", 0);
		Datastore::setValue("txtPointweight/hidden", 0);
		Datastore::setValue("txtPointdisplacement/hidden", 0);
		Datastore::setValue("txtNockweight/hidden", 0);
		Datastore::setValue("txtNockdisplacement/hidden", 0);


	}







	////////////////////////////////////////////////////////////
	// set the values to the base class
	unitstr = this->getValue("txtFF_Frontal/unit").toString();
	v = this->getValue("txtFF_Frontal/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	ff_frontal = v;

	unitstr = this->getValue("txtFF_Cross/unit").toString();
	v = this->getValue("txtFF_Cross/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	ff_cross = v;

	unitstr = this->getValue("txtAreaFrontal/unit").toString();
	v = this->getValue("txtAreaFrontal/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	A_frontal = v;

	unitstr = this->getValue("txtAreaCrossSection/unit").toString();
	v = this->getValue("txtAreaCrossSection/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	A_cross = v;

	unitstr = this->getValue("txtCenterOfPresure/unit").toString();
	v = this->getValue("txtCenterOfPresure/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	cp = v;

	unitstr = this->getValue("txtMomentOfInertia/unit").toString();
	v = this->getValue("txtMomentOfInertia/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	J = v;

	unitstr = this->getValue("txtMass/unit").toString();
	v = this->getValue("txtMass/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m = v;

	//qDebug() << __LINE__ << " " << __FILE__ << this->toString();
	return true;

}

void CPfeil::init() {
	// Defaultvalues for the full (6dof) model:
	//cp = -0.07;
	//J = 0.001;
	//m = 0.027;
	//A_frontal = (0.009 * 0.009 * utility::PI / 4.0);
	//A_cross = 0.5 * 0.009;
	//ff_frontal = 1.0;
	//ff_cross = 1.0;

	double diameter = 0.009; // meter

	// Defaultvalues for the simplified model (cp is zero):
	cp = 0.0;
	J = 0.001;
	m = 0.027;
	A_frontal = (diameter * diameter * utility::PI / 4.0);
	A_cross = A_frontal;
	ff_frontal = 1.0;
	ff_cross = ff_frontal;

	QMap<QString, QVariant> model;
	model["value"] = "simplified";
	model["status"] = Datastore::preset;
	model["unit"] = "";
	model["title"] = "Model";
	this->m_data["txtModel"] = model;


	this->m_data["title"] = QString();

	QMap<QString, QVariant> map;

	///////////////////////////////////
	// The full arrow model (and simplified arrow: shaftdiameter)
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_kilogram";
		map["title"] = QCoreApplication::translate("arrow", "shaft weight");
		map["hidden"] = 1;
		this->m_data["txtShaftweight"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "shaft length");
		map["hidden"] = 1;
		this->m_data["txtShaftlength"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["title"] = QCoreApplication::translate("arrow", "shaft code");
		map["hidden"] = 1;
		this->m_data["txtShaftcode"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "shaft diameter");
		map["hidden"] = 0;
		this->m_data["txtShaftdia"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "shaft thickness");
		map["hidden"] = 1;
		this->m_data["txtShaftthickness"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "shaft displacement");
		map["hidden"] = 1;
		this->m_data["txtShaftdisplacement"] = map;
	}

	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_square_metre";
		map["title"] = QCoreApplication::translate("arrow", "fletches area");
		map["hidden"] = 1;
		this->m_data["txtFletchesarea"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "fletches length");
		map["hidden"] = 1;
		this->m_data["txtFletcheslength"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_kilogram";
		map["title"] = QCoreApplication::translate("arrow", "fletches weight");
		map["hidden"] = 1;
		this->m_data["txtFletchesweight"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "fletches displacement");
		map["hidden"] = 1;
		this->m_data["txtFletchesdisplacement"] = map;
	}

	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_kilogram";
		map["title"] = QCoreApplication::translate("arrow", "point weight");
		map["hidden"] = 1;
		this->m_data["txtPointweight"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "point displacement");
		map["hidden"] = 1;
		this->m_data["txtPointdisplacement"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_kilogram";
		map["title"] = QCoreApplication::translate("arrow", "nock weight");
		map["hidden"] = 1;
		this->m_data["txtNockweight"] = map;
	}
	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "SI_metre";
		map["title"] = QCoreApplication::translate("arrow", "nock displacement");
		map["hidden"] = 1;
		this->m_data["txtNockdisplacement"] = map;
	}


	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["unit"] = "pound_per_square_inch";
		map["title"] = QCoreApplication::translate("arrow", "sectional density");
		map["hidden"] = 1;
		this->m_data["txtSectionaldensity"] = map;
	}

	{
		QMap<QString, QVariant> map;
		map["value"] = "";
		map["status"] = Datastore::preset;
		map["title"] = QCoreApplication::translate("arrow", "ballistic coefficient");
		map["hidden"] = 1;
		this->m_data["txtBallisticcoefficient"] = map;
	}



    ///////////////////////////////////
	// advanced values
	//
	map["value"] = ff_frontal;
	map["status"] = Datastore::preset;
	map["title"] = QCoreApplication::translate("arrow", "formfactor frontal");
	map["hidden"] = 0;
	this->m_data["txtFF_Frontal"] = map;

	map["value"] = ff_cross;
	map["status"] = Datastore::preset;
	map["title"] = "Fromfactor sidewise";
	map["hidden"] = 1;
	this->m_data["txtFF_Cross"] = map;

	map["value"] = A_frontal;
	map["status"] = Datastore::preset;
	map["unit"] = "SI_square_metre";
	map["title"] = QCoreApplication::translate("arrow", "frontal area");
	map["hidden"] = 0;
	this->m_data["txtAreaFrontal"] = map;

	map["value"] = A_cross;
	map["status"] = Datastore::preset;
	map["unit"] = "SI_square_metre";
	map["title"] = QCoreApplication::translate("arrow", "cross section area");
	map["hidden"] = 1;
	this->m_data["txtAreaCrossSection"] = map;

	map["value"] = cp;
	map["status"] = Datastore::preset;
	map["unit"] = "SI_metre";
	map["title"] = QCoreApplication::translate("arrow", "center of presure");
	map["hidden"] = 1;
	this->m_data["txtCenterOfPresure"] = map;

	map["value"] = J;
	map["status"] = Datastore::preset;
	map["unit"] = "SI_kilogram_square_metre";
	map["title"] = QCoreApplication::translate("arrow", "moment of inertia");
	map["hidden"] = 1;
	this->m_data["txtMomentOfInertia"] = map;

	map["value"] = m;
	map["status"] = Datastore::preset;
	map["unit"] = "SI_kilogram";
	map["title"] = QCoreApplication::translate("arrow", "mass");
	map["hidden"] = 0;
	this->m_data["txtMass"] = map;

}



QString CPfeil::toString()
{
	QDomDocument domDocument("arrow");
	QDomElement dom_arrow;
	domDocument.setContent(
			QString("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"));
	QDomElement root = domDocument.createElement("arrowmatcher");
	root.setAttribute("version", "2.0");
	domDocument.appendChild(root);
	dom_arrow = domDocument.createElement("arrow");
	this->toDomElement(dom_arrow);
	root.appendChild(dom_arrow);
	qDebug() << QString("dom Arrow=") << domDocument.toString(4);


	return domDocument.toString(4);
}
int CPfeil::toDomElement(QDomElement &dom_arrow)
{
	arrowmatcher_file af;
	QDomElement dom_element;
	QDomText dom_text;
	QDomDocument domDocument = dom_arrow.ownerDocument();



	dom_element = domDocument.createElement("guid");
	dom_text = domDocument.createTextNode(this->getGuid().toString());
	dom_element.appendChild(dom_text);
	dom_arrow.appendChild(dom_element);

	dom_element = domDocument.createElement("map");
	dom_element.setAttribute("name", "data");
	af.writeMap(dom_element, domDocument, this->m_data);
	dom_arrow.appendChild(dom_element);


	return 0;
}



int CPfeil::fromDomElement(const QDomElement &dom_arrow) {
	arrowmatcher_file af;

	CPfeil temp(QUuid(dom_arrow.firstChildElement("guid").text()));

	QDomElement dom_element = dom_arrow.firstChildElement("map");
	while (!dom_element.isNull()) {
		if(dom_element.attribute("name") == QString("data")){
			af.readMap(temp.m_data, dom_element);
		}
		dom_element = dom_arrow.nextSiblingElement("map");

	}

	*this = temp;
	return 0;

}
int CPfeil::fromString(const QString xmlstr ) {
	QDomDocument domDocument("arrow");
	QDomElement dom_arrow;
	domDocument.setContent(xmlstr);
	QDomElement root = domDocument.documentElement ();
	if( ! root.isElement()) {
		return 1;
	}
	dom_arrow = root.firstChildElement("arrow");
	if( ! dom_arrow.isElement()) {
		return 2;
	}

	this->fromDomElement(dom_arrow);
	//qDebug() << "arrow xmlstr=" << xmlstr;
	//qDebug() << "arrow fromString=" << this->toString();
	return 0;
}
