/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef UNITCONVERTER_H
#define UNITCONVERTER_H

#include <QString>
#include <QList>
#include <QObject>
#include <QListWidgetItem>
#include "Unit.h"

class UnitSystem {
public:
	Unit::UnitSystem m_system;
	QString m_name;

};

class UnitConverter: public QObject {
Q_OBJECT
public:
	UnitConverter();
	UnitConverter(const UnitConverter &unitconv) : QObject() {
		*this = unitconv;
	}

	QList<Unit> m_units;
	Unit::UnitSystem m_system;
	bool convert(double & value, QString dest_id, QString src_id);
	//
	/**
	* convert a value to the SI unit
	*
	*  @param   value  this will be converted to the SI unit, for example: 1 yard is convertet to the SI metre
	*  @param   src_id    the id of the unit: for example: "yard" or "inch"
	*
	*  @return  true or false if it fails
	*
	*/
	bool convert_to_SI(double & value, QString src_id);
	//
	/**
	* convert a value from the SI unit to another
	*
	*  @param   value    must be in a SI unit and is convertet to yard or inch from metre
	*  @param   dest_id    the id of the unit: for example: "yard" or "inch"
	*
	*  @return   true or false if it fails
	*
	*/
	bool convert_from_SI(double & value, QString dest_id);
	//
	/**
	* return the Unit of a unittype
	*
	*  @param   unit by reference, unittype
	*
	*  @return  true or false if it fails
	*
	*/
	bool get_unit(Unit & unit, QString unittype);
	//
	bool convert_to_current_system(double & dest_value, Unit & dest_unit,
			double src_value, QString src_unit_id);
	bool get_current_unit_system(Unit & dest, QString src_unit_id);
	QString get_curr_unit_sym(QString src_unit_id);
	double get_curr_unit_val(double value, QString src_unit_id);

	QList<Unit> get_units(QString unit_id);

	UnitConverter & operator=(const UnitConverter & conv) {
		if (this == &conv)
			return *this;
		m_units = conv.m_units;
		m_system = conv.m_system;
		return *this;

	}

	QList<UnitSystem> get_list_UnitSystem();
};

#endif //UNITCONVERTER_H
