/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "arrowmatcherdoc.h"
#include "arrowmatcherview.h"
#include "arrowmatcher_file.h"

#include <QMessageBox>
#include <QTextStream>
#include <functional>
#include <algorithm>
#include <QDebug>

ArrowmatcherDoc::ArrowmatcherDoc(QWidget *parent) :
	QObject(parent) {
	this->modified = true;
	m_crossbow_list_model = new CrossbowTreeModel(this);
	m_arrow_list_model = new ArrowTreeModel(this);
	m_arrow_list_model->m_unitconverter = &m_unitconverter;
	this->newDocument();
	connect(m_crossbow_list_model, SIGNAL(dataChanged()), this,
			SLOT(dataChanged()));
	connect(m_arrow_list_model, SIGNAL(dataChanged()), this,
			SLOT(dataChanged()));
	//qDebug() << __LINE__ << " " << __FILE__ << m_Arrow.toString();

}

ArrowmatcherDoc::~ArrowmatcherDoc() {
}

bool ArrowmatcherDoc::saveModified() {
	bool completed = true;
	return completed;
}

void ArrowmatcherDoc::closeDocument() {
	deleteContents();
}

bool ArrowmatcherDoc::newDocument() {
	modified = true;
	m_filename = QString();

	m_Angle = 0.0;
	m_Range = 10.0;
	m_windangle = GRAD_TO_RAD(90.0);
	m_windspeed = 0.0;

	m_diagram_max_range = 100.0;// for the diagram
	m_diagram_max_angle = 200.0;// for the diagram
	m_Anglelist.clear();
	m_Targetcircles.clear();

	//diagram_appearance da;
	//m_diagram_appeareance = da;

	this->m_crossbow_list_model->clear();

	this->m_arrow_list_model->clear();

	//this->m_arrow_tree_model->clear();
	//qDebug() << __LINE__ << " " << __FILE__ << m_Arrow.toString();
	m_arrow_list_model->appendRow(m_Arrow);
	m_crossbow_list_model->appendRow(m_Crossbow);
	//this->m_arrow_list_model->appendRow(m_Arrow);


	return true;
}

bool ArrowmatcherDoc::openDocument(const QString& fileName, const char *format /*=0*/) {

	if (fileName.isEmpty())
		return false;

	QFile file(fileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
		QMessageBox::warning(0, tr("arrowmatcher"), tr(
				"Cannot read file %1:\n%2.") .arg(fileName) .arg(
				file.errorString()));
		return false;
	}
	arrowmatcher_file af;
	af.read(&file, this);

	m_filename = fileName;
	modified = false;
	return true;
}

bool ArrowmatcherDoc::saveDocument(const QString& filename, const char *format /*=0*/) {

	QFile file(filename);
	if (!file.open(QIODevice::WriteOnly | QFile::Text)) {
		QMessageBox::warning(0, tr("arrowmatcher"), tr(
				"Cannot read file %1:\n%2.") .arg(filename) .arg(
				file.errorString()));
		return false;
	}
	arrowmatcher_file af;
	af.write(&file, this);

	m_filename = filename;
	modified = false;
	return true;
}

void ArrowmatcherDoc::deleteContents() {
	m_crossbow_list_model->clear();
	m_arrow_list_model->clear();
	m_Angle = 0.0;
	m_Range = 10.0;
	modified = false;
	m_filename = QString();
}

void ArrowmatcherDoc::dataChanged() {
	std::cout << "ArrowmatcherDoc::dataChanged" << std::endl;
	modified = true;

}

int ArrowmatcherDoc::calc_all_setting_curves() {
	m_curve_targetpoint_sight_1.clear();
	m_curve_targetpoint_crossbow_1.clear();
	m_curve_velocity_1.clear();
	m_curve_time_1.clear();

	this->calc_setting_curve(this->m_curve_targetpoint_sight,
			m_curve_targetpoint_crossbow, m_curve_velocity, m_curve_time, this->m_Angle);
	//QVector<QPoint>::Iterator iter_points;
	m_max_range_calculated = false;
	if (this->m_curve_targetpoint_sight.maxX() < this->m_diagram_max_range) {
		flightparam fp;
		Ballistic::flightparam_init(fp, this->m_Arrow);
		Ballistic::flightparam_gravity(fp, this->m_Angle);
		Ballistic::flightparam_wind(fp, this->m_windangle, this->m_windspeed);
		SETTING sight_setting; // Setting of the sight
		arw::flight_snapshot beginparam, endparam;
		int err = this->m_Crossbow.calc_max_range(sight_setting.setting,
				beginparam, endparam, fp);
		if (err == 0) {
			this->m_max_range = endparam.s.x;
			/// @TODO Winkel sollte ueber den Geschwindigkeitsvector berechnet werden:
			this->m_angle_max_range = beginparam.phi.z;
			m_max_range_calculated = true;

		}

	}

	for (int i = 0; i < this->m_Anglelist.size(); i++) {
		akima_curve curve_sight;
		akima_curve curve_crossbow;
		akima_curve curve_velocity;
		aspline curve_time;
		this->calc_setting_curve(curve_sight, curve_crossbow, curve_velocity, curve_time,
				this->m_Anglelist[i]);
		this->m_curve_targetpoint_sight_1.push_back(curve_sight);
		this->m_curve_targetpoint_crossbow_1.push_back(curve_crossbow);
		this->m_curve_velocity_1.push_back(curve_velocity);
		this->m_curve_time_1.push_back(curve_time);

	}
	return 0;
}

int ArrowmatcherDoc::calc_setting_curve(akima_curve &curve, double angle) {
	curve.clear();

	double range = 0.5;
	int err = 0;
	flight_snapshot endparam, beginparam;

	flightparam fp;
	Ballistic::flightparam_init(fp, this->m_Arrow);
	Ballistic::flightparam_gravity(fp, angle);
	Ballistic::flightparam_wind(fp, this->m_windangle, this->m_windspeed);

	double stepsize = 1.0;

	clock_t before;
	double elapsed;
	before = clock();
	SETTING sight_setting; // Setting of the sight

	bool bmax_range = false;
	bool bone_true_point = false;
	for (range = 0.5; range < (this->m_diagram_max_range + stepsize); range
			+= stepsize) ////Querbalken mit Entfernungsangabe zeichnen
	{
		err = this->m_Crossbow.sight_setting(sight_setting.setting, endparam,
				beginparam, range, fp);
		if (err) // '** Fehlercode
		{
			break;
		}
		sight_setting.range = range;
		curve.insertpoint(range, sight_setting.setting);

		stepsize = range * 0.1;
		if (stepsize < 0.5)
			stepsize = 0.5;
		if (stepsize > 30.0)
			stepsize = 30.0;

	}
	// asume that the max range is inside the diagram area:
	if (range < this->m_diagram_max_range) {
		err = this->m_Crossbow.calc_max_range(sight_setting.setting,
				beginparam, endparam, fp);
		if (err == 0) {
			sight_setting.range = endparam.s.x;
			curve.insertpoint(endparam.s.x, sight_setting.setting);
		}
	}
	curve.calcspline();
	if (curve.err)
		return 1;

	double r = curve.maxX();
	return 0;

}

int ArrowmatcherDoc::calc_setting_curve(
		akima_curve &curve_targetpoint_from_sight,
		akima_curve &curve_targetpoint_from_crossbow,
		akima_curve & curve_velocity_range,
		aspline & curve_time_range, double angle) {

	curve_targetpoint_from_sight.clear();
	curve_targetpoint_from_crossbow.clear();
	curve_time_range.clear();
	curve_velocity_range.clear();

	double range = 0.5;
	int err = 0;
	flight_snapshot endparam, beginparam;

	flightparam fp;
	Ballistic::flightparam_init(fp, this->m_Arrow);
	Ballistic::flightparam_gravity(fp, angle);
	Ballistic::flightparam_wind(fp, this->m_windangle, this->m_windspeed);

	double stepsize = 1.0;

	clock_t before;
	double elapsed;
	before = clock();
	SETTING targetpoint_sight_range;
	SETTING targetpoint_crossbow_range;
	SETTING velocity_range;
	double energy;
	double velocity;

	bool bmax_range = false;
	bool bone_true_point = false;
	for (range = 0.5; range < (this->m_diagram_max_range + stepsize); range
			+= stepsize) ////Querbalken mit Entfernungsangabe zeichnen
	{
		err = this->m_Crossbow.sight_setting(targetpoint_sight_range.setting,
				targetpoint_crossbow_range.setting, endparam, beginparam,
				range, fp);
		if (err) // '** Fehlercode
		{
			break;
		}
		targetpoint_sight_range.range = range;
		targetpoint_crossbow_range.range = range;
		velocity_range.range = range;

		curve_targetpoint_from_sight.insertpoint(range,
				targetpoint_sight_range.setting);
		curve_targetpoint_from_crossbow.insertpoint(range,
				targetpoint_crossbow_range.setting);
		curve_velocity_range.insertpoint(range, endparam.v);

		curve_time_range.insertpoint(range, endparam.t);

		stepsize = range * 0.1;
		if (stepsize < 0.5)
			stepsize = 0.5;
		if (stepsize > 30.0)
			stepsize = 30.0;

	}
	// asume that the max range is inside the diagram area:
	if (range < this->m_diagram_max_range) {
		err = this->m_Crossbow.calc_max_range(targetpoint_sight_range.setting,
				beginparam, endparam, fp);
		if (err == 0) {
			targetpoint_sight_range.range = endparam.s.x;
			curve_targetpoint_from_sight.insertpoint(endparam.s.x,
					targetpoint_sight_range.setting);

			targetpoint_crossbow_range.range = endparam.s.x;
			curve_targetpoint_from_crossbow.insertpoint(endparam.s.x,
					targetpoint_crossbow_range.setting);

			curve_velocity_range.insertpoint(endparam.s.x, endparam.v);
			curve_time_range.insertpoint(endparam.s.x, endparam.t);
		}

	}
	curve_targetpoint_from_sight.calcspline();
	if (curve_targetpoint_from_sight.err)
		return 1;
	curve_targetpoint_from_crossbow.calcspline();
	if (curve_targetpoint_from_crossbow.err)
		return 1;

	curve_velocity_range.calcspline();
	curve_time_range.calcspline();

	return 0;

}

bool ArrowmatcherDoc::update_arrow(CPfeil &a) {
	bool updated = false;
	if (m_Arrow == a) {
		m_Arrow = a;
		modified = true;
		updated = true;
	}
	if (m_arrow_list_model->setItem(a)) {
		modified = true;
		updated = true;
	}
	//this->m_arrow_tree_model->setItem(a);
	return updated;
}
bool ArrowmatcherDoc::update_crossbow(CArmbrust &a) {
	bool updated = false;
	if (m_Crossbow == a) {
		m_Crossbow = a;
		modified = true;
		updated = true;
	}
	if (m_crossbow_list_model->setItem(a)) {
		modified = true;
		updated = true;
	}
	return updated;
}
