/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ARROWMATCHERDOC_H
#define ARROWMATCHERDOC_H

#include <QString>
#include <QList>
#include <QUrl>
#include <QFile>

#include "view.h"
#include "akima_curve.h"
#include "ArrowTreeModel.h"
#include "CrossbowTreeModel.h"
#include "UnitConverter.h"
#include "Armbrust.h"

struct SETTING {
	/** sight setting from sight origin */
	utility::vector_3d setting;
	/** range from global origin */
	double range;

};

// forward declaration of the Arrowmatcher classes
class ArrowmatcherView;

/**	ArrowmatcherDoc provides a model
 */
class ArrowmatcherDoc: public QObject {
Q_OBJECT
public:
	/** Constructor for the fileclass of the application */
	ArrowmatcherDoc(QWidget *parent);
	/** Destructor for the fileclass of the application */
	~ArrowmatcherDoc();

	/** sets the modified flag for the document after a modifying action on the view connected to the document.*/
	void setModified(bool _m = true) {
		modified = _m;
	}
	;
	/** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
	bool isModified() {
		return modified;
	}
	;
	/** "save modified" - asks the user for saving if the document is modified */
	bool saveModified();
	/** deletes the document's contents */
	void deleteContents();
	/** initializes the document generally */
	bool newDocument();
	/** closes the acutal document */
	void closeDocument();
	/** loads the document by filename and format and emits the updateViews() signal */
	bool openDocument(const QString& url, const char *format = 0);
	/** saves the document under filename and format.*/
	bool saveDocument(const QString& url, const char *format = 0);
	/** contains the parameter for the diagrams */
	//diagram_appearance  m_diagram_appearance;
	int calc_all_setting_curves();

	int calc_setting_curve(akima_curve &curve, double angle);
	int calc_setting_curve(akima_curve &curve_targetpoint_from_sight,
			akima_curve &curve_targetpoint_from_crossbow,
			akima_curve & curve_velocity_range,
			aspline & curve_time_range, double angle);

	bool update_arrow(CPfeil &a);
	bool update_crossbow(CArmbrust &a);

public slots:
	/** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
	 * As this view normally repaints itself, it is excluded from the paintEvent.
	 */
	//void slotUpdateAllViews(QWidget *sender);
	void dataChanged();

public:

	/** the aktual Crossbow for the calculation */
	CArmbrust m_Crossbow;
	/**  the aktual Arrow for the calculation  */
	CPfeil m_Arrow;

	//crossbow_list_model *m_crossbow_list_model;
	ArrowTreeModel *m_arrow_list_model;
	CrossbowTreeModel *m_crossbow_list_model;


	/** range in Meter */
	double m_Range;
	/** Angle between the line of sight and the horizontal plane in rad*/
	double m_Angle;
	/** Windspeed in m/s */
	double m_windspeed;
	/** Winddirection in rad
	 * a direction in x-axis is zero */
	double m_windangle;

	/** list for more then one Angle for the diagram */
	std::vector<double> m_Anglelist;
	/** max range of the diagramm (0 - max. Range) */
	double m_diagram_max_range;
	/** max setting value of the vertical direction of the diagram */
	double m_diagram_max_angle;
	/** list with all targetdiameters (the user can define this) */
	std::vector<double> m_Targetcircles;

	/** setting parameter for the diagram appearence */
	//diagram_appearance m_diagram_appeareance;

	/** the modified flag of the current document */
	bool modified;
	QString m_filename;

	akima_curve m_curve_targetpoint_sight;
	std::vector<akima_curve> m_curve_targetpoint_sight_1;

	akima_curve m_curve_targetpoint_crossbow;
	std::vector<akima_curve> m_curve_targetpoint_crossbow_1;

	akima_curve m_curve_velocity;
	std::vector<akima_curve> m_curve_velocity_1;

	/** Solution curves for different ranges */
	aspline m_curve_time;
	std::vector<aspline> m_curve_time_1;
	//aspline m_energy_range;
	//std::vector<aspline> m_energy_range_1;

	double m_max_range;
	double m_angle_max_range;
	bool m_max_range_calculated;

	UnitConverter m_unitconverter;

};

#endif // ARROWMATCHERDOC_H
