/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * aspline.h
 *  Created on: 19.06.2009
 */

#ifndef ASPLINE_H_
#define ASPLINE_H_

#include <ballistic/utility.h>
using namespace arw;

class aspline {
	double *x, *y; /* the values to interpolate */
	int n; // count of the points including the first and the last 2 points, which are used for the calculation.
	int n_memory; // memory size for points (size for double)
	//double *dx, *dy; /* x[i+1]-x[i], resp. y[i+1]-y[i] */
	double *t; /* the slopes */
	double *C, *D; /* the coefficients */
	bool not_calculatet;// new point was added or a point was changed
	//int max_control_points;
	void computelimits(int s, int e, double *llimit, double *ulimit);

public:
	bool err;
	aspline();
	aspline(const aspline &s);
	~aspline();
	aspline & operator=(const aspline & s);
	void clear();

	int insertpoint(double xv, double yv);
	int calcspline();
	int spline(double &yv, double xv);// calc the y-value for x
	/** @return number of control points */
	int numPoints() {
		if (n <= 4)
			return 0;
		return n - 4;
	}
	;
	/**
	 * returns the x-value of a  control point
	 * @param Num index of control point
	 * @return y-value of control point
	 */
	double getX(int Num) {
		if (n <= 4)
			return 0.0;
		if (Num > (n - 3))
			return 0.0;
		return x[Num + 2];

	}
	/**
	 * returns the y-value of a  control point
	 * @param Num index of control point
	 * @return y-value of control point
	 */
	double getY(int Num) {
		if (n <= 4)
			return 0.0;
		if (Num > (n - 3))
			return 0.0;
		return y[Num + 2];
	}

	double minX() {
		if (not_calculatet == true) {
			return 0.0;
		}
		return x[2];

	}
	;
	double maxX() {
		if (not_calculatet == true) {
			return 0.0;
		}
		return x[n - 3];
	}
	;
	bool not_calculated(){
		return not_calculatet;
	};

};

#endif /* ASPLINE_H_ */
