/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "crossbows_arrows_settingsdlg.h"
#include "armbrustdlg.h"
#include <ballistic/utility.h>
#include "mainwindow.h"
#include "aspline.h"
#include "arrowmatcher_file.h"

#include <QMessageBox>
#include <QPrinter>
#include <QString>
#include <QMenu>
#include <QPoint>
#include <QContextMenuEvent>
#include <QCursor>
#include <QTableView>
#include <QTreeView>
#include <QPushButton>
#include <QVBoxLayout>
#include <QLineEdit>
#include <QLabel>
#include <QDoubleValidator>
#include <QTreeWidgetItem>
#include <QClipboard>
#include "DlgArrow.h"
#include "UnitInputWidget.h"
#include <QDebug>

#include "ArrowTreeModel.h"

using namespace arw;

void crossbows_arrows_settingsdlg::update(ArrowmatcherDoc *doc) {
	/** @todo Der richtige Pfeil, und die richtige Armbrust muß noch selectiert werden */
	m_pdoc = doc;
	if (m_pdoc == NULL)
		return;

	bool ok;
	double val;
	val = this->txtEntfernung->getSIValue(ok);
	if( ! ok) {
		Unit u;
		m_pdoc->m_unitconverter.get_current_unit_system(u, "SI_metre");
		//qDebug() << m_pdoc->m_unitconverter.get_curr_unit_sym("SI_metre");
		this->txtEntfernung->setValue(10.0, u.m_type_id, &(m_pdoc->m_unitconverter), UnitInputWidget::preset);
	}

	val = this->txtZielwinkel->getSIValue(ok);
	if( ! ok) {
		Unit u;
		m_pdoc->m_unitconverter.get_current_unit_system(u, "degree");
		this->txtZielwinkel->setValue(0.0, u.m_type_id, &(m_pdoc->m_unitconverter), UnitInputWidget::preset);
	}

	val = this->txtWindSpeed->getSIValue(ok);
	if( ! ok) {
		Unit u;
		m_pdoc->m_unitconverter.get_current_unit_system(u, "kilometre_per_hour");
		this->txtWindSpeed->setValue(0.0, u.m_type_id, &(m_pdoc->m_unitconverter), UnitInputWidget::preset);
	}

	val = this->txtWindAngle->getSIValue(ok);
	if( ! ok) {
		Unit u;
		m_pdoc->m_unitconverter.get_current_unit_system(u, "degree");
		val = 90.0;
		m_pdoc->m_unitconverter.convert(val, u.m_type_id, "degree");
		this->txtWindAngle->setValue(val, u.m_type_id, &(m_pdoc->m_unitconverter), UnitInputWidget::preset);
	}

	QString str;
	str = tr("Windspeed");
	this->lblWindspeed->setText(str);
	str = tr("Range");
	this->lblRange->setText(str);


	if (trvCrossbowlist->model() == NULL) {
		trvCrossbowlist->setModel(m_pdoc->m_crossbow_list_model);
		QItemSelectionModel *selectionModel_crossbow = new QItemSelectionModel(
				m_pdoc->m_crossbow_list_model);
		connect(
				selectionModel_crossbow,
				SIGNAL(currentChanged ( const QModelIndex &, const QModelIndex & )),
				this,
				SLOT(on_lstArmbrustliste_currentChanged ( const QModelIndex &, const QModelIndex & )));
		trvCrossbowlist->setSelectionModel(selectionModel_crossbow);
	}
	if (trvArrowlist->model() == NULL) {
		trvArrowlist->setModel(m_pdoc->m_arrow_list_model);
		QItemSelectionModel *selectionModel_arrow = new QItemSelectionModel(
				m_pdoc->m_arrow_list_model);
		trvArrowlist->setSelectionModel(selectionModel_arrow);
		connect(
				selectionModel_arrow,
				SIGNAL(currentChanged ( const QModelIndex &, const QModelIndex & )),
				this,
				SLOT(on_lstPfeilliste_currentChanged ( const QModelIndex &, const QModelIndex & )));

	}

	QModelIndexList select_crossbow = m_pdoc->m_crossbow_list_model->match(QModelIndex(), 0, m_pdoc->m_Crossbow.getGuid().toString());
	if( ! select_crossbow.isEmpty()){
		//int row = select_crossbow.at(0).row();
		//trvCrossbowlist->setEnabled(false);
		//trvCrossbowlist->selectionModel()->setCurrentIndex(select_crossbow.at(0), QItemSelectionModel::Rows);
		//trvCrossbowlist->blockSignals(true);
		//trvCrossbowlist->setCurrentIndex(select_crossbow.at(0));
		//trvCrossbowlist->blockSignals(false);
		//trvCrossbowlist->setEnabled(true);
	}
	//trvCrossbowlist->resizeRowsToContents();

}

crossbows_arrows_settingsdlg::crossbows_arrows_settingsdlg(QWidget *parent) :
	QWidget(parent) {
	m_pdoc = NULL;
}
crossbows_arrows_settingsdlg::~crossbows_arrows_settingsdlg() {
}

void crossbows_arrows_settingsdlg::cmdOk_click() {
	if (m_pdoc == NULL) {
		return;
	}
	//std::cout << m_pdoc->m_Arrow.getGuid().toString().latin1() << endl;
	Unit unit_range;
	Unit unit_speed;
	bool ok;


	m_pdoc->m_Range = this->txtEntfernung->getSIValue(ok);
	m_pdoc->m_Angle = this->txtZielwinkel->getSIValue(ok);
	m_pdoc->m_windspeed = this->txtWindSpeed->getSIValue(ok);
	m_pdoc->m_windangle = this->txtWindAngle->getSIValue(ok);

	m_pdoc->calc_all_setting_curves();
	//this->m_pControler->slotUpdateAllViews(this);
	this->m_pControler->slotUpdateAllViews(NULL);
}

/** must call bevor show or exec */
void crossbows_arrows_settingsdlg::addView(MainWindow* mainwindow) {

	this->m_pControler = mainwindow;
	mainwindow->m_content_col_left->addTab(this, tr("Setting"));

	cmd_Ok = new QPushButton(tr("Execute"));
	cmd_Ok->setMaximumWidth(200);
	cmd_Ok->setIcon(QPixmap(":/toolbar/button_ok.png"));
	trvCrossbowlist = new QTreeView(this);
	trvCrossbowlist->setSelectionBehavior(QAbstractItemView::SelectRows);
	trvCrossbowlist->setSelectionMode(QAbstractItemView::SingleSelection);
	trvCrossbowlist->setContextMenuPolicy(Qt::CustomContextMenu);
	QPixmap *bg_image = new QPixmap(":/toolbar/bg_crossbow.png");
	this->SetBackgroundImage(trvCrossbowlist, bg_image);
	QPixmap *bg_image_pfeil = new QPixmap(":/toolbar/bg_arrow.png");
	trvArrowlist = new QTreeView(this);
	this->trvArrowlist->setSelectionBehavior(QAbstractItemView::SelectRows);
	this->trvArrowlist->setSelectionMode(QAbstractItemView::SingleSelection);
	trvArrowlist->setContextMenuPolicy(Qt::CustomContextMenu);
	this->SetBackgroundImage(trvArrowlist, bg_image_pfeil);

	this->txtEntfernung = new UnitInputWidget();
	this->txtEntfernung->setUnitstyle(UnitInputWidget::symbol);
	this->txtZielwinkel = new UnitInputWidget();
	this->txtZielwinkel->setToolTip(tr("Uphill or downhill in degree"));
	this->txtZielwinkel->setUnitstyle(UnitInputWidget::symbol);
	this->txtWindAngle = new UnitInputWidget();
	this->txtWindAngle->setUnitstyle(UnitInputWidget::symbol);
	this->txtWindSpeed = new UnitInputWidget();
	this->txtWindSpeed->setUnitstyle(UnitInputWidget::symbol);
	this->lblRange = new QLabel(tr("Range"));
	QLabel *lbl_2 = new QLabel(tr("Angle"));
	this->lblWindspeed = new QLabel(tr("Windspeed"));
	QLabel *lbl_4 = new QLabel(tr("Winddirection"));

	QPixmap *image = new QPixmap(":/images/range_wind.png");
	QLabel *lbl_image = new QLabel();
	lbl_image->setPixmap(*image);

	QHBoxLayout *inputfields_image = new QHBoxLayout;
	QVBoxLayout *fields = new QVBoxLayout;
	fields->addWidget(this->lblRange);
	fields->addWidget(txtEntfernung);
	fields->addWidget(lbl_2);
	fields->addWidget(txtZielwinkel);
	fields->addWidget(this->lblWindspeed);
	fields->addWidget(this->txtWindSpeed);
	fields->addWidget(lbl_4);
	fields->addWidget(this->txtWindAngle);
	fields->addWidget(cmd_Ok);
	inputfields_image->addLayout(fields);
	inputfields_image->addWidget(lbl_image);


	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(trvArrowlist);
	layout->addWidget(trvCrossbowlist);
	layout->addLayout(inputfields_image);

	//layout->addWidget(cmd_Ok);
	this->setLayout(layout);

	QDoubleValidator *v = new QDoubleValidator(this);
	txtEntfernung->setValidator(v);
	v = new QDoubleValidator(this);
	txtZielwinkel->setValidator(v);
	v = new QDoubleValidator(this);
	txtWindSpeed->setValidator(v);
	v = new QDoubleValidator(this);
	txtWindAngle->setValidator(v);

	connect(trvCrossbowlist, SIGNAL(customContextMenuRequested(QPoint)), this,
			SLOT(showCrossbowListItemMenu(QPoint)));
	connect(trvArrowlist, SIGNAL(customContextMenuRequested(QPoint)), this,
			SLOT(showArrowListItemMenu(QPoint)));
	connect(cmd_Ok, SIGNAL(clicked()), this, SLOT(cmdOk_click()));

	QAction *SettingsCurrentArrow = new QAction(QIcon(":/toolbar/tlb_arrow"),
			tr("Current &Arrow"), this);
	connect(SettingsCurrentArrow, SIGNAL(triggered()), this,
			SLOT(slotModifyCurrentArrow()));

	QAction *SettingsCurrentCrossbow = new QAction(QIcon(
			":/toolbar/tlb_crossbow"), tr("Current &Crossbow"), this);
	connect(SettingsCurrentCrossbow, SIGNAL(triggered()), this,
			SLOT(slotModifyCurrentCrossbow()));

	mainwindow->m_editToolBar->addAction(SettingsCurrentArrow);
	mainwindow->m_editToolBar->addAction(SettingsCurrentCrossbow);

	action_Crossbow_new = new QAction(this);
	action_Crossbow_new->setText(tr("New"));
	action_Crossbow_edit = new QAction(this);
	action_Crossbow_edit->setText(tr("Edit"));
	action_Crossbow_delete = new QAction(this);
	action_Crossbow_delete->setText(tr("Delete"));
	action_Crossbow_copy = new QAction(this);
	action_Crossbow_copy->setText(tr("Copy"));
	action_Crossbow_paste = new QAction(this);
	action_Crossbow_paste->setText(tr("Paste"));

	popup_crossbowlist = new QMenu(this);
	popup_crossbowlist->addAction(action_Crossbow_new);
	popup_crossbowlist->addAction(action_Crossbow_edit);
	popup_crossbowlist->addAction(action_Crossbow_delete);
	popup_crossbowlist->addAction(action_Crossbow_copy);
	popup_crossbowlist->addAction(action_Crossbow_paste);

	action_Arrow_new = new QAction(this);
	action_Arrow_new->setText(tr("New"));
	action_Arrow_edit = new QAction(this);
	action_Arrow_edit->setText(tr("Edit"));
	action_Arrow_delete = new QAction(this);
	action_Arrow_delete->setText(tr("Delete"));
	action_Arrow_copy = new QAction(this);
	action_Arrow_copy->setText(tr("Copy"));
	action_Arrow_paste = new QAction(this);
	action_Arrow_paste->setText(tr("Paste"));

	popup_arrowlist = new QMenu(this);
	popup_arrowlist->addAction(action_Arrow_new);
	popup_arrowlist->addAction(action_Arrow_edit);
	popup_arrowlist->addAction(action_Arrow_delete);
	popup_arrowlist->addAction(action_Arrow_copy);
	popup_arrowlist->addAction(action_Arrow_paste);

}

void crossbows_arrows_settingsdlg::showCrossbowListItemMenu(const QPoint &pos)
{
		if (m_pdoc == NULL)
			return;
		QTreeView *treeWidget = qobject_cast<QTreeView*> (sender());

		if (!treeWidget) {
			std::cout << "kein Widget" << std::endl;
			return;
		}
		QModelIndex index = treeWidget->indexAt(pos);
		std::cout << "gültige Armbrust" << std::endl;

		QAction *action = popup_crossbowlist->exec(
				treeWidget->viewport()->mapToGlobal(pos));
		if (action == action_Crossbow_edit) {
			if (!index.isValid()) {
				QMessageBox::information(this, tr("Info"), tr(
						"no selected Crossbow"), QMessageBox::Ok);
				return;
			}
			CArmbrust* a = m_pdoc->m_crossbow_list_model->getItem(index);
			if (!a) {
				QMessageBox::information(this, tr("Info"), tr(
						"no selected Crossbow"), QMessageBox::Ok);
				return;
			}

			CArmbrust p = *a;
			ArmbrustDlg *dlg = new ArmbrustDlg(p, m_pdoc->m_arrow_list_model, this);

			if (dlg->exec() == QDialog::Accepted) {
				m_pdoc->update_crossbow(p);
			}
			this->cmdOk_click();

		} else if (action == action_Crossbow_new) {
			CArmbrust p;
			m_pdoc->m_crossbow_list_model->appendRow(p);

		} else if (action == action_Crossbow_delete) {
			if (!index.isValid()) {
				std::cout << "kein gültiger Crossbowindex" << std::endl;
				QMessageBox::information(this, tr("Info"), tr(
						"no selected Crossbow"), QMessageBox::Ok);
				return;
			}
			int row = index.row();
			m_pdoc->m_crossbow_list_model->removeRows(row, 1, index);

		} else if (action == action_Crossbow_copy) {
			CArmbrust* a = m_pdoc->m_crossbow_list_model->getItem(index);
			if (!a) {
				std::cout << "keine Pfeil" << std::endl;
				QMessageBox::information(this, tr("Info"), tr("no selected Crossbow"),
						QMessageBox::Ok);
				return;
			}
			QClipboard *clipboard = QApplication::clipboard();
			//QString originalText = clipboard->text();
			clipboard->setText(a->toString());

		} else if (action == action_Crossbow_paste) {
			CArmbrust *a = NULL;
			if (!index.isValid()) {
				QModelIndex new_index = m_pdoc->m_crossbow_list_model->index(0,0, QModelIndex() );// root item
				m_pdoc->m_crossbow_list_model->insertRows(0, 1, new_index);
				new_index = m_pdoc->m_crossbow_list_model->index(0, 1, QModelIndex() );// fetch the index of the first item
				a = m_pdoc->m_crossbow_list_model->getItem(new_index);

			}else{
				a = m_pdoc->m_crossbow_list_model->getItem(index);
			}
			if (!a) {
				std::cout << "keine Pfeil" << std::endl;
				QMessageBox::information(this, tr("Info"), tr("no selected Crossbow"),
						QMessageBox::Ok);
				return;
			}
			CArmbrust p;
			const QClipboard *clipboard = QApplication::clipboard();
			const QMimeData *mimeData = clipboard->mimeData();

			if ( ! mimeData->hasText()) {
				QMessageBox::information(this, tr("Info"), tr("wrong format"),
						QMessageBox::Ok);
				return;
			}
			if( 0 != p.fromString(mimeData->text())) {
				QMessageBox::information(this, tr("Info"), tr("cannot parse text from clipboard"),
						QMessageBox::Ok);
				return;
			}

			a->take_data(p);
			m_pdoc->update_crossbow(*a);
			this->cmdOk_click();
		}


}
void crossbows_arrows_settingsdlg::showArrowListItemMenu(const QPoint &pos)
{
	std::cout << "Popup Pfeil" << std::endl;

	if (m_pdoc == NULL)
		return;
	QTreeView *treeWidget = qobject_cast<QTreeView*> (sender());

	if (!treeWidget) {
		std::cout << "kein Widget" << std::endl;
		return;
	}
	QModelIndex index = treeWidget->indexAt(pos);

	QAction *action = popup_arrowlist->exec(
			treeWidget->viewport()->mapToGlobal(pos));
	if (action == action_Arrow_edit) {
		if (!index.isValid()) {
			std::cout << "kein gültiger Pfeilindex" << std::endl;
			QMessageBox::information(this, tr("Info"), tr("no selected Arrow"),
					QMessageBox::Ok);
			return;
		}
		CPfeil* a = m_pdoc->m_arrow_list_model->getItem(index);
		if (!a) {
			std::cout << "keine Pfeil" << std::endl;
			QMessageBox::information(this, tr("Info"), tr("no selected Arrow"),
					QMessageBox::Ok);
			return;
		}
		DlgArrow *dlg = new DlgArrow(this);
		dlg->init(*a, m_pdoc->m_unitconverter);
		if (dlg->exec() == QDialog::Accepted) {
			m_pdoc->update_arrow(dlg->m_arrow);
		}
		this->cmdOk_click();

	} else if (action == action_Arrow_new) {

		int row = index.row();
		m_pdoc->m_arrow_list_model->insertRows(row, 1, index);

	} else if (action == action_Arrow_delete) {
		if (!index.isValid()) {
			std::cout << "kein gültiger Pfeilindex" << std::endl;
			QMessageBox::information(this, tr("Info"), tr("no selected Arrow"),
					QMessageBox::Ok);
			return;
		}
		int row = index.row();
		m_pdoc->m_arrow_list_model->removeRows(row, 1, index);


	} else if (action == action_Arrow_copy) {
		CPfeil* a = m_pdoc->m_arrow_list_model->getItem(index);
		if (!a) {
			std::cout << "keine Pfeil" << std::endl;
			QMessageBox::information(this, tr("Info"), tr("no selected Arrow"),
					QMessageBox::Ok);
			return;
		}
		QClipboard *clipboard = QApplication::clipboard();
		//QString originalText = clipboard->text();
		clipboard->setText(a->toString());

	} else if (action == action_Arrow_paste) {
		CPfeil *a = NULL;
		if (!index.isValid()) {
			QModelIndex new_index = m_pdoc->m_arrow_list_model->index(0,0, QModelIndex() );// root item
			m_pdoc->m_arrow_list_model->insertRows(0, 1, new_index);
			new_index = m_pdoc->m_arrow_list_model->index(0, 1, QModelIndex() );// fetch the index of the first item
			a = m_pdoc->m_arrow_list_model->getItem(new_index);

		}else{
			a = m_pdoc->m_arrow_list_model->getItem(index);
		}
		if (!a) {
			std::cout << "keine Pfeil" << std::endl;
			QMessageBox::information(this, tr("Info"), tr("no selected Arrow"),
					QMessageBox::Ok);
			return;
		}
		CPfeil p;
		const QClipboard *clipboard = QApplication::clipboard();
		const QMimeData *mimeData = clipboard->mimeData();

		if ( ! mimeData->hasText()) {
			QMessageBox::information(this, tr("Info"), tr("wrong format"),
					QMessageBox::Ok);
			return;
		}
		if( 0 != p.fromString(mimeData->text())) {
			QMessageBox::information(this, tr("Info"), tr("cannot parse text from clipboard"),
					QMessageBox::Ok);
			return;
		}

		a->take_data(p);
		m_pdoc->update_arrow(*a);
		this->cmdOk_click();
	}

}
void crossbows_arrows_settingsdlg::slotModifyCurrentArrow() {

	DlgArrow *dlg = new DlgArrow(this);
	CPfeil p = m_pdoc->m_Arrow;
	dlg->init(m_pdoc->m_Arrow, m_pdoc->m_unitconverter);
	if (dlg->exec() != QDialog::Accepted) {
		return;
	}
	m_pdoc->update_arrow(dlg->m_arrow);
	return;
}
void crossbows_arrows_settingsdlg::slotModifyCurrentCrossbow() {
	std::cout << "Pfeil: currentChanged\n" << std::endl;
	CArmbrust p(m_pdoc->m_Crossbow);
	ArmbrustDlg *dlg = new ArmbrustDlg(p, m_pdoc->m_arrow_list_model, this);
	if (dlg->exec() != QDialog::Accepted) {
		return;
	}
	m_pdoc->update_crossbow(p);
}

/*!
 \fn crossbows_arrows_settingsdlg::SetBackgroundImage ( QWidget *fp_widget, const QPixmap *fp_image )
 found at:
 http://www.qtforum.de/forum/viewtopic.php?t=951
 */
void crossbows_arrows_settingsdlg::SetBackgroundImage(QWidget *fp_widget,
		const QPixmap *fp_image) {
	QPalette newPalette(fp_widget->palette());
	newPalette.setBrush(QPalette::Base, QBrush(*fp_image));
	fp_widget->setPalette(newPalette);
}

void crossbows_arrows_settingsdlg::on_lstPfeilliste_currentChanged(
		const QModelIndex &current, const QModelIndex &previous) {
	//std::cout << "Pfeil: currentChanged\n" << std::endl;

	CPfeil *p = m_pdoc->m_arrow_list_model->getItem(current);
	if (p == NULL) {
		QMessageBox::information(this, tr("Info"), tr("no selected Arrow"),
				QMessageBox::Ok);
		return;

	}
	m_pdoc->m_Arrow = *p;

	//this->m_pControler->slotUpdateAllViews(this);


}
void crossbows_arrows_settingsdlg::on_lstArmbrustliste_currentChanged(
		const QModelIndex &current, const QModelIndex &previous) {
	//std::cout << "Armbrust: currentChanged\n" << std::endl;
	CArmbrust *p = m_pdoc->m_crossbow_list_model->getItem(current);
	if (p == NULL) {
		QMessageBox::information(this, tr("Info"), tr("no selected Crossbow"),
				QMessageBox::Ok);
		return;

	}
	m_pdoc->m_Crossbow = *p;
}
