/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui>
#include "arrowmatcherdoc.h"
#include "UnitConverter.h"
#include <QProcess>
#include <QTemporaryFile>

class QAction;
class QActionGroup;
class QMenu;
class QTextEdit;

class QAbstractItemModel;
class QItemSelectionModel;

class crossbow_list_model;
class QAssistantClient;
class QTabWidget;
/**
 * This MainWindow provides a SDI userinterface. It is made quite similar as this example:
@see http://doc.trolltech.com/4.2/mainwindows-sdi.html

*/
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow();
     /* holds all pointers to widgets, that should be updated if the document is modified */
    QList<View*> *m_pViewList;
    bool views_need_recreate();
    QString m_path2gnuplot;

    // path to the last saved file, this is convinient
    QString m_last_save_path;

    QTabWidget *m_content_col_main;
    QTabWidget *m_content_col_left;

    //UnitConverter m_unitconverter;


protected:
    void closeEvent(QCloseEvent *event);

public slots:
    void slotNewFile();
    void open();
    bool save();
    bool saveAs();
    void about();
    void documentWasModified();
    void openRecentForm();
    void slotHelp();

    /** No descriptions */
    void slotEditSight_Diagram_appearance();

    void slotPlotTrajectory();
    void slotShowPreferencesDlg();
    void slotunitsystemActGroup(QAction* act);

    void helpBrowser_finished(int exitCode,
    		QProcess::ExitStatus exitStatus);
//    CPfeil slotGetCurrentArrow(){
//    	return m_pdoc->m_Arrow;
//    };
public:
    void slotUpdateAllViews(View *sender);
    void addView(View *view);
    void removeView(View *view);

    void init(const QString &fileName);

    QMenu *m_fileMenu;
    QMenu *m_editMenu;
    QMenu *m_helpMenu;
    QToolBar *m_fileToolBar;
    QToolBar *m_editToolBar;

private:
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void readSettings();
    void writeSettings();
    bool maybeSave();
    void loadFile(const QString &fileName);
    bool saveFile(const QString &fileName);
    void setCurrentFile(const QString &fileName);
    QString strippedName(const QString &fullFileName);
    MainWindow *findMainWindow(const QString &fileName);

    QSplitter *m_splitter_main;


    //QTextEdit *textEdit;
    QString curFile;
    bool isUntitled;

    QAction *newAct;
    QAction *openAct;
    QAction *saveAct;
    QAction *saveAsAct;
    QAction *closeAct;
    QAction *exitAct;
    QAction *aboutAct;
    QAction *aboutQtAct;
    QAction *helpAct;
    QAction *helptestAct;

    QActionGroup *unitsystemActGroup;


    QAction *plotTrajectory;



    QAction* SettingsSightDiagram_appearance;
    QAction* SettingsEdit_diagram_scope;
    QAction* SettingsEditTargetcircles;
    QAction* SettingsDiagram_appearance;
    QAction* showPreferencesDlg;
    //QAction* SettingsUnitMetric;
    //QAction* SettingsUnitEnglish;

    QItemSelectionModel *selectionModel;

    QActionGroup *m_recentFilesActions;
    enum { MaxRecentFiles = 10 };

    void updateRecentFileActions();
    void clearRecentFiles();
    QActionGroup *recentFilesActions() const;
    void addRecentFile(const QString &fileName);


    QProcess *m_gnuplot;
    QTemporaryFile *m_gnuplot_io_file;

    bool m_views_need_recreate;
    ArrowmatcherDoc *m_pdoc;
protected:
    QStringList m_recentFiles;
    //QAssistantClient * m_assistantClient;
    QProcess* m_helpBrowser;
};





#endif
