/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "visier_diagram.h"
// application specific includes
#include "arrowmatcherdoc.h"
#include "Armbrust.h"
#include <QMenu>
#include <QPainter>
#include <QTextStream>
#include <QVector>
#include <QMouseEvent>
#include <QPaintEvent>
#include <vector>
#include <list>
#include <string>
#include <QRegion>
#include <QMessageBox>
#include <iostream>
#include <stdio.h>
#include <QLayout>
#include <QtDebug>
#include <algorithm>
#include <map>
#include <ballistic/utility.h>
#include "sightdiagram_appearancedlg.h"

// Für zeitmessung:
#include <time.h>
#include <iostream>

#include "aspline.h"

visier_diagram::visier_diagram(QWidget *parent) :
	QWidget(parent) {
	//std::cout << "visier_diagram konstructor\n";
	//setBackgroundMode(Qt::PaletteBase);
	//setBackgroundColor(Qt::white );
	//m_zoom = 1;//faktor zur vergr�erung des absehens, kann ver�dert werden
	//m_welcheGrafik = Anhaltepunktekurve;
	//m_pan = 0;//gibt die vertikale verschiebung des Bildes an
	m_fontheight = 12;
	m_doc = NULL;
	m_Pixmap = NULL;

	popup_options = new QMenu(this);
	popup_options->addAction(tr("options"), this, SLOT(edit_options()));

}

visier_diagram::~visier_diagram() {
	//std::cout << "visier_diagram destructor\n";
}

ArrowmatcherDoc *visier_diagram::getDocument() const {
	return m_doc;
}

void visier_diagram::paintEvent(QPaintEvent *paintEvent) {
	if (m_Pixmap == NULL)
		return;
	//std::cout << "paintEvent" << std::endl;
	QPainter temp(this);
	temp.drawPixmap(0, 0, *m_Pixmap);

}

void visier_diagram::drawDiagram(QPainter &dc) {
	ArrowmatcherDoc *doc = getDocument();
	if (doc == NULL) {
		return;
	}
	if(doc->m_curve_targetpoint_sight.not_calculated())
		return;

	double dEntfernung;
	int iFehler;
	double dEinstellwert;
	double dIntervall;

	QString s;

	QPen originalPen = dc.pen();
	QFont originalFont = dc.font();
	QBrush originalBrush = dc.brush();
	QFont fontLabelAxis("courier", m_fontheight, QFont::Light);//Beschriftung der beiden Achsen

	QBrush transparentBrush(Qt::NoBrush);
	QPen Absehen(QColor("DarkGrey"), 0, Qt::SolidLine);
	QPen RangeMark(QColor("green"), 0, Qt::SolidLine);// "ForestGreen"
	QPen senkrechteRaster(QColor("LightGrey"), 0, Qt::SolidLine); //helles Grau
	QPen Kurve(QColor("red"), 0, Qt::SolidLine);
	QPen Schriftstift(QColor("black"), 0, Qt::SolidLine);
	QPen red_dash(QColor("red"), 0, Qt::DotLine);
	QPen pen_max_range_mark(QColor("magenta"), 3, Qt::SolidLine);
	QPen pen_Crosshair(QColor("black"), 2, Qt::SolidLine);
	//QBrush      brush_PaleGreen
	QPoint P1;
	QPoint P2;

	QRect test;
	QRect textlabel;//for the values of the diagram-axis
	//QFontInfo info(fontLabelAxis);
	//QRegion diagram_region(Rechteck.left(), Rechteck.top(),
	//                    Rechteck.width(), Rechteck.height(), QRegion::Rectangle);
	bool drawlabel;

	fontLabelAxis.setStyleHint(QFont::Courier, QFont::PreferQuality);
	// invalid values:
	if (doc->m_Angle > utility::PI / 2)
		return;
	if (doc->m_Angle < -utility::PI / 2)
		return;

	//flightparam m_fp;
	Ballistic::flightparam_init(m_fp, doc->m_Arrow);
	Ballistic::flightparam_gravity(m_fp, doc->m_Angle);
	Ballistic::flightparam_wind(m_fp, doc->m_windangle, doc->m_windspeed);

	//dc.setClipRegion(diagram_region);
	//dc.setClipping(true);

	//////////////////////////////
	//Senkrechte Markierungen(Entfernungen) zeichnen mit Beschriftung:
	//////////////////////////////////////////

	dc.setPen(senkrechteRaster);
	//dc.setFont (fontLabelAxis);
	dc.setFont(this->font());

	textlabel.setHeight(dc.fontMetrics().height());// Values for the vertical-axis
	textlabel.setWidth(70);


	//dIntervall = fabs((textlabel.width() / 2.0) / m_dFactor_x);// Mindestabstand der Beschriftungen, umgerechnet in Meter, 10 is a good value for dIntervall and 4(Factor_x
	//utility::calc_interval(dIntervall);


	Unit unit;
	double width_curr_unit;
	doc->m_unitconverter.convert_to_current_system(width_curr_unit, unit, doc->m_diagram_max_range, "SI_metre");
	//qDebug() << "curr system=" << unit.m_type_id;
	double fact_curr_unit = m_diagram_area.width() / width_curr_unit;
	dIntervall = fabs((textlabel.width() / 2.0) / fact_curr_unit);
	this->calc_interval(dIntervall);
	//qDebug() << "intervall=" << dIntervall;
	doc->m_unitconverter.convert_to_SI(dIntervall, unit.m_type_id);
	//qDebug() << "intervall=" << dIntervall;



	drawlabel = true;
	for (dEntfernung = dIntervall; m_dFactor_x * dEntfernung
			< m_diagram_area.width() - 10.0; dEntfernung += dIntervall) ////Querbalken mit Entfernungsangabe zeichnen
	{
		drawlabel = (!drawlabel);
		P1.setX(m_diagram_area.left() + m_dFactor_x * dEntfernung);
		P2.setX(P1.x());
		P1.setY(m_diagram_area.top());
		P2.setY(m_diagram_area.bottom());
		s.setNum(doc->m_unitconverter.get_curr_unit_val(dEntfernung, "SI_metre"), 'g', 4);
		if (!drawlabel)
			senkrechteRaster.setStyle(Qt::DotLine);
		else
			senkrechteRaster.setStyle(Qt::SolidLine);
		dc.setPen(senkrechteRaster);
		//dc.moveTo(P1);
		dc.drawLine(P1, P2);
		if (drawlabel) {
			dc.setPen(Schriftstift);
			textlabel.moveCenter(P1);
			textlabel.translate(0.0, -textlabel.height() / 2.0);
			dc.drawText(textlabel, Qt::AlignCenter, s);
			//dc.drawRect(textlabel);

		}
	}
	s = tr("Range")+ QString(" [") + doc->m_unitconverter.get_curr_unit_sym("SI_metre") + QString("]");
	dc.setPen(Schriftstift);
	//dc.font().setBold(true);
	textlabel = QRect(m_diagram_area.left(), m_window_area.top(),
			m_diagram_area.width(), 20);
	dc.fillRect(textlabel, QBrush(QColor("yellow")));
	//dc.drawRect(textlabel);
	dc.drawText(textlabel, Qt::AlignCenter, s);

	/////////////////////////////////////////////
	// Absehen oder Skaleneinteilung:
	///////////////////////////////////////////
	draw_sight_scale(dc);

	QVector<QPoint>::Iterator iter_points;
	utility::vector_3d setting;

	/////////////////////////////////////
	//Kurven Zeichnen:
	///////////////////////////////
	//textlabel.setWidth(30);
	textlabel.setHeight(dc.fontMetrics().height());// Values for the vertical-axis

	//std::vector<double>::iterator pwinkel;
	for (int i = 0; i < doc->m_Anglelist.size(); i++) {
		QPolygon graph_sight;
		std::vector<SETTING>::iterator iter;
		//QPolygon main_graph_sight_1;
		std::cout << "############ begin #############" << std::endl;
		if(doc->m_curve_targetpoint_sight_1.size() < 1)
			break;
		for (double range = doc->m_curve_targetpoint_sight_1[i].minX(); range
				<= doc->m_curve_targetpoint_sight_1[i].maxX(); range += 0.5) {
			QPoint _P;
			if (doc->m_curve_targetpoint_sight_1[i].spline(setting, range)) {
				// Fehler;
			}
			setting /= setting.x;
			_P.setY(y2diagram(setting));
			_P.setX(m_dFactor_x * range + m_diagram_area.left());
			graph_sight << _P;

		}

		Kurve.setStyle(Qt::DotLine);
		dc.setPen(Kurve);
		QPoint P1;
		P1.setX(0.0);
		QPoint P;

		dc.drawPolyline(graph_sight);

		iter_points = graph_sight.end();
		do {
			iter_points--;
		} while ((iter_points != graph_sight.begin())
				&& (!m_diagram_area.contains(*iter_points)));
		P = *iter_points;

		//label with the angle for the graph (needs the last point of the graph P1 !!)
		s.setNum(doc->m_Anglelist[i] * 180.0 / utility::PI, 'f', 2);
		s.append(trUtf8("°"));
		//dc.setPen(red_dash);
		dc.setPen(Kurve);
		textlabel.setWidth(70);
		textlabel.moveTopLeft(P);
		if (textlabel.bottom() > m_diagram_area.bottom())
			textlabel.moveBottomRight(QPoint(textlabel.right(),
					m_diagram_area.bottom()));
		if (textlabel.right() > m_diagram_area.right())
			textlabel.moveBottomRight(QPoint(m_diagram_area.right(),
					textlabel.bottom()));
		//dc.fillRect(textlabel, QBrush(QColor("#FFC8C8")));
		dc.drawText(textlabel, Qt::AlignRight, s);
		//dc.setPen(QPen(QColor("red"), 0, Qt::SolidLine));
		//dc.drawRect(textlabel);

	}
	Kurve.setStyle(Qt::SolidLine);
	/////////////////////////////////////
	// draw the Main-Graph:
	///////////////////////////////
	dEntfernung = 0.5;
	//vector<setting>::iterator iter;
	dc.setPen(Kurve);
	/////////////////////
	//vector<SETTING>::iterator iter_1;
	QPolygon main_graph_sight_1;
	std::cout << "############ begin #############" << std::endl;
	//double r = doc->m_setting_curve_1[0].maxX();
	for (double range = doc->m_curve_targetpoint_sight.minX(); range
			<= doc->m_curve_targetpoint_sight.maxX(); range += 0.5) {
		QPoint _P;
		if (doc->m_curve_targetpoint_sight.spline(setting, range)) {
			// Fehler;
		}
		setting /= setting.x;
		_P.setY(y2diagram(setting));
		_P.setX(m_dFactor_x * range + m_diagram_area.left());
		main_graph_sight_1 << _P;

	}

	std::cout << "############ end #############" << std::endl;
	dc.drawPolyline(main_graph_sight_1);

	// find last point, which is in the diagram area:
	iter_points = main_graph_sight_1.end();
	if (main_graph_sight_1.size() == 0) {
		s = tr("cannot draw the graph because no points");
		print_errormessage(dc, s);
		return;
	}
	do {
		iter_points--;
	} while ((iter_points != main_graph_sight_1.begin())
			&& (!m_diagram_area.contains(*iter_points)));

	QPoint P = *iter_points;
	s.setNum(doc->m_Angle * 180.0 / utility::PI, 'f', 2);
	s.append(trUtf8("°"));
	dc.setPen(Kurve);
	textlabel.setWidth(70);
	textlabel.moveTopRight(P);
	if (textlabel.bottom() > m_diagram_area.bottom())
		textlabel.moveBottomRight(QPoint(textlabel.right(),
				m_diagram_area.bottom()));
	if (textlabel.right() > m_diagram_area.right())
		textlabel.moveBottomRight(QPoint(m_diagram_area.right(),
				textlabel.bottom()));
	//dc.fillRect(textlabel, QBrush(QColor("#FFC8C8")));
	dc.drawText(textlabel, Qt::AlignRight, s);
	//dc.setPen(Kurve);
	//dc.drawRect(textlabel);

	////////////////////////////////////////////////////////////
	// draw the Range Mark
	/////////////////////////////////////////////////////////////
	flight_snapshot endparam, beginparam;
	//utility::vector_3d setting;
	//double dAlpha;
	//if (doc->m_valid.range) {

	iFehler = doc->m_Crossbow.sight_setting(setting, endparam, beginparam,
			doc->m_Range, m_fp);
	if (iFehler) // '** Fehlercode
	{
		s
				= tr(
						"Cannot calc the setting\nmaybe the range is to big, try a shorter range");
		print_errormessage(dc, s);
	} else {
		//std::sort(doc->m_Targetcircles.begin(),doc->m_Targetcircles.end(),std::greater<double>() );
		//this->m_impactparam = endparam;
		setting /= setting.x;
		QPoint centerP(m_diagram_area.left() + m_dFactor_x * doc->m_Range,
				y2diagram(setting));

		QRect targetcircle;
		dc.setPen(RangeMark);
		// senkrechte Markierung:
		dc.drawLine(QPoint(m_diagram_area.left() + m_dFactor_x * doc->m_Range,
				m_diagram_area.top()), QPoint(m_diagram_area.left()
				+ m_dFactor_x * doc->m_Range, m_diagram_area.bottom()));

		textlabel.setHeight(dc.fontMetrics().height());
		textlabel.setWidth(100);
		textlabel.moveCenter(QPoint(centerP.x(), m_diagram_area.top() - 3*
				textlabel .height() / 2 - 2));

		s.setNum(doc->m_unitconverter.get_curr_unit_val(doc->m_Range, "SI_metre"), 'g', 4);
		dc.drawText(textlabel, Qt::AlignCenter, s);

		std::vector<double>::iterator pd;
		textlabel.moveCenter(QPoint(m_diagram_area.left() + 50.0,
				m_diagram_area.bottom() - 60.0));
		for (pd = doc->m_Targetcircles.begin(); pd
				!= doc->m_Targetcircles.end(); pd++) {
			dc.setPen(RangeMark);
			targetcircle.setWidth(m_dFactor_y * (*pd) * 1000.0 / doc->m_Range);
			targetcircle.setHeight(targetcircle.width());
			targetcircle.moveCenter(centerP);
			dc.drawEllipse(targetcircle);

			dc.setPen(Schriftstift);
			//dc.moveTo(QPoint(centerP.x(), targetcircle.bottom() ));
			textlabel.translate(0, -textlabel.height());
			dc.drawLine(QPoint(centerP.x(), targetcircle.bottom()), QPoint(
					textlabel.right(), textlabel.center().y()));
			s.setNum(*pd * 1000.0, 'f', 1);
			s.append(" mm");
			s = trUtf8("ø") + QString(" ") + s;
			dc.fillRect(textlabel, QBrush(QColor("white")));
			dc.drawText(textlabel, Qt::AlignRight, s);
		}
		dc.setPen(RangeMark);

		// waagrechte Markierung des Visierpunktes:
		dc.drawLine(QPoint(m_diagram_area.right(), centerP.y()), QPoint(
				m_diagram_area.left(), centerP.y()));

		//dc.setPen(Schriftstift);
		textlabel.setHeight(dc.fontMetrics().height());
		textlabel.setWidth(70);

		if (doc->m_Crossbow.m_Zielgeraet.m_iTyp == KIMME_U_KORN) {
//			textlabel.moveCenter(QPoint(m_diagram_area.right() - 110,
//					centerP.y()));
			textlabel.moveCenter(QPoint(20 + textlabel.width() / 2,
					centerP.y()));

			// Setting-value is for the Crosshairs in mm/m, else the real value
			//dc.fillRect(textlabel, QBrush(QColor("PaleGreen")));
			s.setNum(y_setting(setting) / y_setting(
					doc->m_Crossbow.m_Zielgeraet.m_graduation), 'f', 2);
			dc.drawText(textlabel, Qt::AlignRight, s);
		}

	}
	////////////////////////////////////////////////////////////
	// draw the Max Range mark
	/////////////////////////////////////////////////////////////
	if(doc->m_max_range_calculated){
		dc.setPen(pen_max_range_mark);
		// senkrechte Markierung:
		dc.drawLine(QPoint(m_diagram_area.left() + m_dFactor_x * doc->m_max_range,
				m_diagram_area.top()), QPoint(m_diagram_area.left()
				+ m_dFactor_x * doc->m_max_range, m_diagram_area.bottom()));

		textlabel.setHeight(2 * dc.fontMetrics().height() );
		textlabel.setWidth(70);
		textlabel.moveCenter(QPoint(m_diagram_area.left() + m_dFactor_x * doc->m_max_range, m_diagram_area.top() - 2*
				textlabel .height() / 2 - 2));
		s.setNum(doc->m_max_range, 'f', 1);
		QString s_angle;
		s_angle.setNum( (doc->m_angle_max_range * 180.0 / arw::utility::PI) , 'f', 2);
		s += QString("\n") + s_angle + trUtf8("°");
		dc.drawText(textlabel, Qt::AlignCenter, s);


	}


	dc.setPen(Schriftstift);

	dc.drawRect(m_diagram_area);
	//dc.restore();

	QRect frame = m_window_area;
	frame.setWidth(m_window_area.width() - 1);
	frame.setHeight(m_window_area.height() - 1);

	dc.drawRect(frame);
	dc.setPen(originalPen);
	dc.setFont(originalFont);
	dc.setBrush(originalBrush);

}

int visier_diagram::zeichnen(QPainter &dc, QRect &Rechteck) {
	QPen originalPen = dc.pen();
	QFont originalFont = dc.font();
	QBrush originalBrush = dc.brush();
	QFont fontText("courier", m_fontheight, QFont::Light);//Entfernungsangeben der Querbalken
	QPen black_thin_solid(QColor("black"), 0, Qt::SolidLine);
	//int zellheight = dc.fontMetrics().lineSpacing();
	//std::cout << "zeichnen" << __LINE__ << std::endl;

	dc.setPen(black_thin_solid);
	dc.setFont(fontText);
	dc.setBackgroundMode(Qt::TransparentMode);//OpaqueMode  bringt beim ausdrucken nicht
	//dc.setBackgroundColor(QColor(255, 255, 255));//Hintergrundfarbe von Schrift etc. ...

	//QRect diagram(0, 0, width(), height());//diagram(0,0,970,550 );

	drawDiagram(dc);

	dc.setPen(originalPen);
	dc.setFont(originalFont);
	dc.setBrush(originalBrush);

	//cout << "int visier_diagram::zeichnen(QPainter &dc, QRect &Rechteck)" << endl;
	return 0;
}



/** No descriptions */
void visier_diagram::mousePressEvent(QMouseEvent *e) {
	if (e->button() == Qt::RightButton) {
		popup_options->popup(e->globalPos());
		//edit_options();
		return;
	}

	if (e->button() != Qt::LeftButton) {
		//parentWidget()->mousePressEvent(e);
		return;
	}
	//	if (m_widget_values != NULL) {
	//		delete m_widget_values;
	//		m_widget_values = NULL;
	//	}
	//	if (m_widget_line != NULL) {
	//		delete m_widget_line;
	//		m_widget_line = NULL;
	//	}
	//
	//	//std::cout << "visier_diagram::MousePressEvent()\n";
	//	if (!m_diagram_area.contains(e->x(), e->y())) {
	//		return;
	//	}
	//	m_widget_values = new QLabel(this, "Another");
	//	m_widget_values->setTextFormat(Qt::RichText);
	//	m_widget_values->resize(200, 200);
	//	m_widget_line = new QLabel(this, "line");
	//	m_widget_line->resize(2, 500);
	//	m_widget_line->setPaletteBackgroundColor(QColor(255, 100, 100));
	//	m_widget_line->show();
	//	m_widget_values->show();
	this->mouseMoveEvent(e);

}
/** No descriptions */
void visier_diagram::mouseReleaseEvent(QMouseEvent *e) {
	if (e->button() != Qt::LeftButton) {
		return;
	}
	//	//std::cout << "visier_diagram::MouseReleaseEvent()\n";
	//	delete m_widget_values;
	//	m_widget_values = 0;
	//	delete m_widget_line;
	//	m_widget_line = 0;

}
/** No descriptions */
void visier_diagram::mouseMoveEvent(QMouseEvent *e) {
	//std::cout << "visier_diagram::MouseMoveEvent()\n";
	//	if (m_widget_values == NULL && m_widget_line == NULL) {
	//		return;
	//	}
	//	if (!m_diagram_area.contains(e->x(), e->y())) {
	//		return;
	//	}
	//
	//	int mx = e->x();
	//	int my = e->y();
	//
	//	double dEntfernung = (mx - m_diagram_area.left()) / m_dFactor_x;
	//	//P2.setX( m_diagram_area.left() + m_dFactor_x * (*iter_flight_snapshot).fs.xs );
	//
	////	vector<setting>::iterator iter = m_main_graph.begin();
	////	for (iter = m_main_graph.begin(); iter != m_main_graph.end(); iter++) {
	////		if ((*iter).impact_fs.s.x >= dEntfernung)
	////			break;
	////	}
	//
	//	QString msg;
	//	//msg ="<table><tr>";
	//	double dSpindelsteigung =
	//			m_doc->m_Crossbow.m_Zielgeraet.dGetSpindelsteigung();
	//	double setting;
	////	if (m_doc->m_Crossbow.m_Zielgeraet.iGetTyp() != KIMME_U_KORN)
	////		setting = (*iter).setting;
	////	else
	////		setting = (*iter).setting / dSpindelsteigung;
	////
	////	msg.sprintf(QString("<table cellpadding=0 cellspacing=0>") + "<tr><td>"
	////			+ tr("Range") + " [m]:</td><td>%01.4f</td></tr>" + "<tr><td>" + tr(
	////			"Setting") + " [mm/m]: </td><td>%01.4f</td></tr>" + "<tr><td>"
	////			+ tr("Duration") + " [s]: </td><td>%01.4f</td></tr>" + "<tr><td>"
	////			+ tr("Angle") + " [grad]: </td><td>%01.4f</td></tr>" + "<tr><td>"
	////			+ tr("Drop") + " [m]: </td><td>%01.4f</td></tr>" + "</table>",
	////			(*iter).impact_fs.s.x, setting, (*iter).impact_fs.t, (*iter).angle
	////					* 180.0 / utility::PI, (*iter).drop);
	////	//msg+="</tr></table>";
	////	m_widget_values->setText(msg);
	////
	////	if (mx + m_widget_values->width() > this->width())
	////		m_widget_values->move(mx - m_widget_values->width(), my);
	////	else
	////		m_widget_values->move(mx + 2, my);
	////
	////	m_widget_line->move(mx, 50);
	////	m_widget_line->show();
	////	m_widget_values->show();
}

void visier_diagram::print_errormessage(QPainter &dc, QString & s) {
	QPen originalPen = dc.pen();
	QFont originalFont = dc.font();
	QBrush originalBrush = dc.brush();

	QRect textlabel;

	textlabel.setHeight(dc.fontMetrics().lineSpacing() * 5);
	textlabel.setWidth(500);
	textlabel.moveCenter(this->rect().center());
	dc.setPen(QPen(QColor("red"), 0, Qt::SolidLine));
	dc.fillRect(textlabel, QBrush(QColor("yellow")));
	dc.drawText(textlabel, Qt::AlignCenter, s);
	dc.drawRect(textlabel);

	dc.setPen(originalPen);
	dc.setBrush(originalBrush);
	dc.setFont(originalFont);

}

void visier_diagram::edit_options() {
	ArrowmatcherDoc *doc = getDocument();
	sightdiagram_appearanceDlg *dlg = new sightdiagram_appearanceDlg(this);
	dlg->init(&doc->m_Targetcircles, &doc->m_Anglelist,
			doc->m_diagram_max_range, doc->m_diagram_max_angle);
	if (dlg->exec() == QDialog::Accepted) {

	}
}

/*!
 \fn visier_diagram::recreate(ArrowmatcherDoc *doc)
 */
void visier_diagram::recreate(ArrowmatcherDoc *doc) {

	//qDebug() << "visier_diagram::recreate" ;

	m_doc = doc;
	if (m_Pixmap == NULL)
		m_Pixmap = new QPixmap(width(), height());

	QRect r(0, 0, width(), height());
	m_window_area = QRect(0, 0, width(), height());
	m_diagram_area = QRect(QPoint(r.left() + 150, r.top() + 80),
			r.bottomRight());
	m_dFactor_x = m_diagram_area.width() / doc->m_diagram_max_range;
	m_dFactor_y = (m_diagram_area.height()) / doc->m_diagram_max_angle;
	m_pan_y = 10.0e999;
	m_Pixmap->fill(Qt::white);

	m_fontheight = 12;//12
	QPainter paint(m_Pixmap);
	//paint.begin(this);
	//paint.setBackgroundColor(QColor(255, 255, 255));//Hintergrundfarbe von Schrift etc. ...
	paint.setWindow(0, 0, 100, 100);
	paint.setViewport(0, 0, 100, 100);

	zeichnen(paint, r);
	update();

}
double visier_diagram::y2diagram(double setting) {
	return m_dFactor_y * setting + m_dFactor_y
			* m_doc->m_Crossbow.get_sight_zero_direction().y * (-1000.0)
			+ m_pan_y + m_diagram_area.top();
}
double visier_diagram::y2diagram(utility::vector_3d setting) {
	return y2diagram(setting.y);
}
double visier_diagram::y_setting(utility::vector_3d setting) {
	return setting.y;
}
double visier_diagram::x2diagram(double range) {
	//return m_diagram_area.left() + dFactor_x * range;
	return m_diagram_area.left() + m_dFactor_x * range;
}

/** calculate the Interval for the scale */
double visier_diagram::calc_interval(double &dIntervall) {
	dIntervall = fabs(dIntervall);// nur positiven Wert
	double fact = 0.00000000000001;
	double startvalue = 0.00000000000001;
	while (startvalue < dIntervall) {
		startvalue *= 100.0;
	}
	startvalue /= 100.0;
	for (fact = startvalue; fact < 100000000000000.0; fact *= 10.0) {

		if (dIntervall < 1.0 * fact) {
			dIntervall = 1.0 * fact;
			return dIntervall;
		} else if (dIntervall < 2.0 * fact) {
			dIntervall = 2.0 * fact;
			return dIntervall;
		} else if (dIntervall < 5.0 * fact) {
			dIntervall = 5.0 * fact;
			return dIntervall;
		}
	}
	return fact;

}

